@@ -1,4 +1,4 @@
-/* $Id: thunar-renamer-dialog.c 26421 2007-12-02 13:46:28Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2005-2007 Benedikt Meurer <benny@xfce.org>
  *
@@ -28,6 +28,8 @@
 #include <string.h>
 #endif
 
+#include <libxfce4ui/libxfce4ui.h>
+
 #include <thunar/thunar-abstract-dialog.h>
 #include <thunar/thunar-application.h>
 #include <thunar/thunar-dialogs.h>
@@ -56,13 +58,12 @@ enum
 /* Identifiers for DnD target types */
 enum
 {
+  TARGET_GTK_TREE_MODEL_ROW,
   TARGET_TEXT_URI_LIST,
 };
 
 
 
-static void     thunar_renamer_dialog_class_init          (ThunarRenamerDialogClass *klass);
-static void     thunar_renamer_dialog_init                (ThunarRenamerDialog      *renamer_dialog);
 static void     thunar_renamer_dialog_dispose             (GObject                  *object);
 static void     thunar_renamer_dialog_finalize            (GObject                  *object);
 static void     thunar_renamer_dialog_get_property        (GObject                  *object,
@@ -79,7 +80,7 @@ static void     thunar_renamer_dialog_response            (GtkDialog
                                                            gint                      response);
 static void     thunar_renamer_dialog_context_menu        (ThunarRenamerDialog      *renamer_dialog,
                                                            guint                     button,
-                                                           guint32                   time);
+                                                           guint32                   timestamp);
 static void     thunar_renamer_dialog_help                (ThunarRenamerDialog      *renamer_dialog);
 static void     thunar_renamer_dialog_save                (ThunarRenamerDialog      *renamer_dialog);
 static void     thunar_renamer_dialog_action_add_files    (GtkAction                *action,
@@ -101,17 +102,23 @@ static void     thunar_renamer_dialog_drag_data_received  (GtkWidget
                                                            gint                      y,
                                                            GtkSelectionData         *selection_data,
                                                            guint                     info,
-                                                           guint                     time,
+                                                           guint                     timestamp,
                                                            ThunarRenamerDialog      *renamer_dialog);
 static void     thunar_renamer_dialog_drag_leave          (GtkWidget                *tree_view,
                                                            GdkDragContext           *context,
-                                                           guint                     time,
+                                                           guint                     timestamp,
                                                            ThunarRenamerDialog      *renamer_dialog);
 static gboolean thunar_renamer_dialog_drag_motion         (GtkWidget                *tree_view,
                                                            GdkDragContext           *context,
                                                            gint                      x,
                                                            gint                      y,
-                                                           guint                     time,
+                                                           guint                     timestamp,
+                                                           ThunarRenamerDialog      *renamer_dialog);
+static gboolean thunar_renamer_dialog_drag_drop           (GtkWidget                *tree_view,
+                                                           GdkDragContext           *context,
+                                                           gint                      x,
+                                                           gint                      y,
+                                                           guint                     timestamp,
                                                            ThunarRenamerDialog      *renamer_dialog);
 static void     thunar_renamer_dialog_notify_page         (GtkNotebook              *notebook,
                                                            GParamSpec               *pspec,
@@ -182,41 +189,18 @@ static const GtkActionEntry action_entries[] =
 /* Target types for dropping to the tree view */
 static const GtkTargetEntry drop_targets[] =
 {
-  { "text/uri-list", 0, TARGET_TEXT_URI_LIST, },
+  { "GTK_TREE_MODEL_ROW", GTK_TARGET_SAME_WIDGET, TARGET_GTK_TREE_MODEL_ROW },
+  { "text/uri-list", 0, TARGET_TEXT_URI_LIST }
+};
+
+/* Target types for dragging files in the tree view */
+static const GtkTargetEntry drag_targets[] = {
+  { "GTK_TREE_MODEL_ROW", GTK_TARGET_SAME_WIDGET, TARGET_GTK_TREE_MODEL_ROW }
 };
 
 
 
-static GObjectClass *thunar_renamer_dialog_parent_class;
-
-
-
-GType
-thunar_renamer_dialog_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarRenamerDialogClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunar_renamer_dialog_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarRenamerDialog),
-        0,
-        (GInstanceInitFunc) thunar_renamer_dialog_init,
-        NULL,
-      };
-
-      type = g_type_register_static (THUNAR_TYPE_ABSTRACT_DIALOG, I_("ThunarRenamerDialog"), &info, 0);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE (ThunarRenamerDialog, thunar_renamer_dialog, THUNAR_TYPE_ABSTRACT_DIALOG)
 
 
 
@@ -227,9 +211,6 @@ thunar_renamer_dialog_class_init (ThunarRenamerDialogClass *klass)
   GtkWidgetClass *gtkwidget_class;
   GObjectClass   *gobject_class;
 
-  /* determine the parent type class */
-  thunar_renamer_dialog_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->dispose = thunar_renamer_dialog_dispose;
   gobject_class->finalize = thunar_renamer_dialog_finalize;
@@ -335,7 +316,7 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
   GList                  *renamers = NULL;
   GList                  *lp;
   gint                    active = 0;
-  gint                    n;
+  guint                   n;
 
   /* allocate a new bulk rename model */
   renamer_dialog->model = thunar_renamer_model_new ();
@@ -368,7 +349,7 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
   button = gtk_dialog_add_button (GTK_DIALOG (renamer_dialog), _("_Rename Files"), GTK_RESPONSE_ACCEPT);
   exo_binding_new (G_OBJECT (renamer_dialog->model), "can-rename", G_OBJECT (button), "sensitive");
   gtk_dialog_set_default_response (GTK_DIALOG (renamer_dialog), GTK_RESPONSE_ACCEPT);
-  thunar_gtk_widget_set_tooltip (button, _("Click here to actually rename the files listed above to their new names."));
+  gtk_widget_set_tooltip_text (button, _("Click here to actually rename the files listed above to their new names."));
 
   /* setup the action group for this dialog */
   renamer_dialog->action_group = gtk_action_group_new ("ThunarRenamerDialog");
@@ -409,10 +390,12 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
   gtk_widget_show (swin);
 
   /* create the tree view */
-  renamer_dialog->tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (renamer_dialog->model));
+  renamer_dialog->tree_view = exo_tree_view_new ();
+  gtk_tree_view_set_model (GTK_TREE_VIEW (renamer_dialog->tree_view), GTK_TREE_MODEL (renamer_dialog->model));
   gtk_tree_view_set_search_column (GTK_TREE_VIEW (renamer_dialog->tree_view), THUNAR_RENAMER_MODEL_COLUMN_OLDNAME);
   gtk_tree_view_set_fixed_height_mode (GTK_TREE_VIEW (renamer_dialog->tree_view), TRUE);
   gtk_tree_view_set_enable_search (GTK_TREE_VIEW (renamer_dialog->tree_view), TRUE);
+  gtk_tree_view_set_rubber_banding (GTK_TREE_VIEW (renamer_dialog->tree_view), TRUE);
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (renamer_dialog->tree_view), TRUE);
   g_signal_connect (G_OBJECT (renamer_dialog->tree_view), "button-press-event", G_CALLBACK (thunar_renamer_dialog_button_press_event), renamer_dialog);
   g_signal_connect (G_OBJECT (renamer_dialog->tree_view), "popup-menu", G_CALLBACK (thunar_renamer_dialog_popup_menu), renamer_dialog);
@@ -505,7 +488,7 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
 
       /* add a "Help" button */
       button = gtk_button_new ();
-      thunar_gtk_widget_set_tooltip (button, _("Click here to view the documentation for the selected rename operation."));
+      gtk_widget_set_tooltip_text (button, _("Click here to view the documentation for the selected rename operation."));
       g_signal_connect_swapped (G_OBJECT (button), "clicked", G_CALLBACK (thunar_renamer_dialog_help), renamer_dialog);
       gtk_box_pack_start (GTK_BOX (rbox), button, FALSE, FALSE, 0);
       gtk_widget_show (button);
@@ -581,12 +564,14 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
 
       /* close the config handle */
       xfce_rc_close (rc);
-  
+
       /* setup drop support for the tree view */
-      gtk_drag_dest_set (renamer_dialog->tree_view, GTK_DEST_DEFAULT_DROP, drop_targets, G_N_ELEMENTS (drop_targets), GDK_ACTION_COPY);
+      gtk_drag_dest_set (renamer_dialog->tree_view, 0, drop_targets, G_N_ELEMENTS (drop_targets), GDK_ACTION_COPY | GDK_ACTION_MOVE);
+      gtk_drag_source_set (renamer_dialog->tree_view, GDK_BUTTON1_MASK, drag_targets, G_N_ELEMENTS (drag_targets), GDK_ACTION_MOVE);
       g_signal_connect (G_OBJECT (renamer_dialog->tree_view), "drag-data-received", G_CALLBACK (thunar_renamer_dialog_drag_data_received), renamer_dialog);
       g_signal_connect (G_OBJECT (renamer_dialog->tree_view), "drag-leave", G_CALLBACK (thunar_renamer_dialog_drag_leave), renamer_dialog);
       g_signal_connect (G_OBJECT (renamer_dialog->tree_view), "drag-motion", G_CALLBACK (thunar_renamer_dialog_drag_motion), renamer_dialog);
+      g_signal_connect (G_OBJECT (renamer_dialog->tree_view), "drag-drop", G_CALLBACK (thunar_renamer_dialog_drag_drop), renamer_dialog);
 
       /* setup the tree selection */
       selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (renamer_dialog->tree_view));
@@ -610,7 +595,7 @@ thunar_renamer_dialog_init (ThunarRenamerDialog *renamer_dialog)
       gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
       gtk_widget_show (image);
 
-      /* TRANSLATORS: You can test this string by temporarily removing thunar-sbr.* from $libdir/thunarx-1/,
+      /* TRANSLATORS: You can test this string by temporarily removing thunar-sbr.* from $libdir/thunarx-2/,
        *              and opening the multi rename dialog by selecting multiple files and pressing F2.
        */
       label = gtk_label_new (_("No renamer modules were found on your system. Please check your\n"
@@ -848,7 +833,7 @@ thunar_renamer_dialog_response (GtkDialog *dialog,
 static void
 thunar_renamer_dialog_context_menu (ThunarRenamerDialog *renamer_dialog,
                                     guint                button,
-                                    guint32              time)
+                                    guint32              timestamp)
 {
   GtkActionGroup *renamer_actions = NULL;
   ThunarxRenamer *renamer;
@@ -904,7 +889,7 @@ thunar_renamer_dialog_context_menu (ThunarRenamerDialog *renamer_dialog,
 
   /* run the menu on the dialog's screen */
   menu = gtk_ui_manager_get_widget (renamer_dialog->ui_manager, "/file-context-menu");
-  thunar_gtk_menu_run (GTK_MENU (menu), GTK_WIDGET (renamer_dialog), NULL, NULL, button, time);
+  thunar_gtk_menu_run (GTK_MENU (menu), GTK_WIDGET (renamer_dialog), NULL, NULL, button, timestamp);
 
   /* remove the previously merge items from the UI manager */
   if (G_UNLIKELY (renamer_merge_id != 0))
@@ -945,7 +930,8 @@ thunar_renamer_dialog_help (ThunarRenamerDialog *renamer_dialog)
   if (G_LIKELY (help_url == NULL))
     {
       /* open the general documentation if no specific URL */
-      thunar_dialogs_show_help (GTK_WIDGET (renamer_dialog), "advanced-topics", "to-bulk-rename-files");
+      xfce_dialog_show_help (GTK_WINDOW (renamer_dialog), "thunar",
+                             "advanced-topics", "to-bulk-rename-files");
     }
   else
     {
@@ -953,7 +939,7 @@ thunar_renamer_dialog_help (ThunarRenamerDialog *renamer_dialog)
       screen = gtk_widget_get_screen (GTK_WIDGET (renamer_dialog));
 
       /* try to launch the specific URL */
-      if (!exo_url_show_on_screen (help_url, NULL, screen, &error))
+      if (!gtk_show_uri (screen, help_url, gtk_get_current_event_time (), &error))
         {
           /* tell the user that we failed */
           thunar_dialogs_show_error (GTK_WIDGET (renamer_dialog), error, _("Failed to open the documentation browser"));
@@ -1308,12 +1294,16 @@ thunar_renamer_dialog_drag_data_received (GtkWidget           *tree_view,
                                           gint                 y,
                                           GtkSelectionData    *selection_data,
                                           guint                info,
-                                          guint                time,
+                                          guint                timestamp,
                                           ThunarRenamerDialog *renamer_dialog)
 {
-  ThunarFile *file;
-  GList      *path_list;
-  GList      *lp;
+  ThunarFile              *file;
+  GList                   *file_list;
+  GList                   *lp;
+  GtkTreeModel            *model;
+  GtkTreePath             *path;
+  GtkTreeViewDropPosition  drop_pos;
+  gint                     position = -1;
 
   _thunar_return_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog));
   _thunar_return_if_fail (GTK_IS_TREE_VIEW (tree_view));
@@ -1321,29 +1311,55 @@ thunar_renamer_dialog_drag_data_received (GtkWidget           *tree_view,
   /* we only accept text/uri-list drops with format 8 and atleast one byte of data */
   if (info == TARGET_TEXT_URI_LIST && selection_data->format == 8 && selection_data->length > 0)
     {
-      /* determine the path list from the selection_data */
-      path_list = thunar_vfs_path_list_from_string ((const gchar *) selection_data->data, NULL);
+      /* determine the renamer model */
+      model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));
 
-      /* add all paths to the model */
-      for (lp = path_list; lp != NULL; lp = lp->next)
+      /* get the drop position, if no path is foud we append the files */
+      if (gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (tree_view), x, y, &path, &drop_pos))
         {
-          /* determine the file for the path */
-          file = thunar_file_get_for_path (lp->data, NULL);
-          if (G_LIKELY (file != NULL))
-            {
-              /* append the file to the model */
-              thunar_renamer_model_append (renamer_dialog->model, file);
+          /* get the position of the path */
+          position = gtk_tree_path_get_indices (path)[0];
+          gtk_tree_path_free (path);
 
-              /* release the file */
-              g_object_unref (G_OBJECT (file));
-            }
+          /* advance the offset if we drop after the item */
+          if (drop_pos == GTK_TREE_VIEW_DROP_AFTER || drop_pos == GTK_TREE_VIEW_DROP_INTO_OR_AFTER)
+            position++;
 
-          /* release the path */
-          thunar_vfs_path_unref (lp->data);
+          /* append if we've reached the end of the model */
+          if (position >= gtk_tree_model_iter_n_children (model, NULL))
+            position = -1;
         }
 
+      /* determine the file list from the selection_data */
+      file_list = thunar_g_file_list_new_from_string ((const gchar *) selection_data->data);
+
+      /* add all paths to the model */
+      for (lp = file_list; lp != NULL; lp = lp->next)
+        {
+          /* determine the file for the path */
+          file = thunar_file_get (lp->data, NULL);
+          if (G_LIKELY (file != NULL))
+            {
+              /* insert the file in the model */
+              thunar_renamer_model_insert (renamer_dialog->model, file, position);
+
+              /* advance the offset if we do not append, so the drop order is consistent */
+              if (position != -1)
+                position++;
+
+              /* release the file */
+              g_object_unref (file);
+            }
+
+          /* release the GFile */
+          g_object_unref (lp->data);
+        }
+
+      /* finish the drag */
+      gtk_drag_finish (context, (file_list != NULL), FALSE, timestamp);
+
       /* release the list */
-      g_list_free (path_list);
+      g_list_free (file_list);
     }
 
   /* stop the emission of the "drag-data-received" signal */
@@ -1355,7 +1371,7 @@ thunar_renamer_dialog_drag_data_received (GtkWidget           *tree_view,
 static void
 thunar_renamer_dialog_drag_leave (GtkWidget           *tree_view,
                                   GdkDragContext      *context,
-                                  guint                time,
+                                  guint                timestamp,
                                   ThunarRenamerDialog *renamer_dialog)
 {
   _thunar_return_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog));
@@ -1379,19 +1395,33 @@ thunar_renamer_dialog_drag_motion (GtkWidget           *tree_view,
                                    GdkDragContext      *context,
                                    gint                 x,
                                    gint                 y,
-                                   guint                time,
+                                   guint                timestamp,
                                    ThunarRenamerDialog *renamer_dialog)
 {
-  GdkAtom target;
+  GdkAtom                  target;
+  GtkTreeViewDropPosition  position;
+  GtkTreePath             *path;
+  GdkDragAction            action;
+  GtkTreeModel            *model;
+  gint                     n_rows;
 
   _thunar_return_val_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog), FALSE);
   _thunar_return_val_if_fail (GTK_IS_TREE_VIEW (tree_view), FALSE);
 
   /* determine the drop target */
   target = gtk_drag_dest_find_target (tree_view, context, NULL);
-  if (G_UNLIKELY (target != gdk_atom_intern ("text/uri-list", FALSE)))
+  if (target == gdk_atom_intern_static_string ("text/uri-list"))
+    {
+      /* set the drag action for uris */
+      action = GDK_ACTION_COPY;
+    }
+  else if (target == gdk_atom_intern_static_string ("GTK_TREE_MODEL_ROW"))
+    {
+      /* set the drag action for tree rows */
+      action = GDK_ACTION_MOVE;
+    }
+  else
     {
-      /* unhighlight the widget if highlighted */
       if (renamer_dialog->drag_highlighted)
         {
           /* we use the tree view parent (the scrolled window),
@@ -1404,22 +1434,110 @@ thunar_renamer_dialog_drag_motion (GtkWidget           *tree_view,
       /* we cannot handle the drop */
       return FALSE;
     }
+      
+  /* compute the drop position */
+  if (gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (tree_view), x, y, &path, &position))
+    {
+      /* we can only drop before or after an item */
+      if (position == GTK_TREE_VIEW_DROP_BEFORE || position == GTK_TREE_VIEW_DROP_INTO_OR_BEFORE)
+        position = GTK_TREE_VIEW_DROP_BEFORE;
+      else
+        position = GTK_TREE_VIEW_DROP_AFTER;
+    }
   else
     {
-      /* highlight the widget */
-      if (!renamer_dialog->drag_highlighted)
+      /* append the item if there are rows in the model */
+      model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));
+      n_rows = gtk_tree_model_iter_n_children (model, NULL);
+
+      if (G_LIKELY (n_rows > 0))
         {
-          /* we use the tree view parent (the scrolled window),
-           * as the drag_highlight doesn't work for tree views.
-           */
-          gtk_drag_highlight (tree_view->parent);
-          renamer_dialog->drag_highlighted = TRUE;
+          /* get the last tree path in the model */
+          path = gtk_tree_path_new_from_indices (n_rows - 1, -1);
+          position = GTK_TREE_VIEW_DROP_AFTER;
+        }
+    }
+
+  if (G_LIKELY (path != NULL))
+    {
+      /* highlight the appropriate row */
+      gtk_tree_view_set_drag_dest_row (GTK_TREE_VIEW (tree_view), path, position);
+      gtk_tree_path_free (path);
+    }
+  else if (!renamer_dialog->drag_highlighted)
+    {
+      /* highlight the parent */
+      gtk_drag_highlight (tree_view->parent);
+      renamer_dialog->drag_highlighted = TRUE;
+    }
+
+  /* we can handle the drop */
+  gdk_drag_status (context, action, timestamp);
+  return TRUE;
+}
+
+
+
+static gboolean
+thunar_renamer_dialog_drag_drop (GtkWidget           *tree_view,
+                                 GdkDragContext      *context,
+                                 gint                 x,
+                                 gint                 y,
+                                 guint                timestamp,
+                                 ThunarRenamerDialog *renamer_dialog)
+{
+  GdkAtom                  target;
+  GtkTreeSelection        *selection;
+  GtkTreePath             *dst_path;
+  GtkTreeModel            *model;
+  GtkTreeViewDropPosition  drop_pos;
+  gint                     position = -1;
+  GList                   *rows;
+  
+  _thunar_return_val_if_fail (THUNAR_IS_RENAMER_DIALOG (renamer_dialog), FALSE);
+  _thunar_return_val_if_fail (GTK_IS_TREE_VIEW (tree_view), FALSE);
+  
+  /* determine the drop target */
+  target = gtk_drag_dest_find_target (tree_view, context, NULL);
+  if (G_LIKELY (target == gdk_atom_intern_static_string ("text/uri-list")))
+    {
+      /* request the data, we call gtk_drag_finish() later */
+      gtk_drag_get_data (tree_view, context, target, timestamp);
+    }
+  else if (target == gdk_atom_intern_static_string ("GTK_TREE_MODEL_ROW"))
+    {
+      selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
+      rows = gtk_tree_selection_get_selected_rows (selection, &model);
+      if (G_LIKELY (rows != NULL))
+        {
+          /* get the drop position, if no path is found we append the file */
+          if (gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (tree_view), x, y, &dst_path, &drop_pos))
+            {
+              /* get the position of the path */
+              position = gtk_tree_path_get_indices (dst_path)[0];
+              gtk_tree_path_free (dst_path);
+
+              /* advance the offset if we drop after the item */
+              if (drop_pos == GTK_TREE_VIEW_DROP_AFTER || drop_pos == GTK_TREE_VIEW_DROP_INTO_OR_AFTER)
+                position++;
+            }
+
+          /* perform the move */
+          thunar_renamer_model_reorder (renamer_dialog->model, rows, position);
+          g_list_foreach (rows, (GFunc) gtk_tree_path_free, NULL);
+          g_list_free (rows);
         }
 
-      /* we can handle the drop */
-      gdk_drag_status (context, GDK_ACTION_COPY, time);
-      return TRUE;
+      /* finish the dnd operation */
+      gtk_drag_finish (context, TRUE, FALSE, timestamp);
     }
+  else
+    {
+      /* we cannot handle the drop */
+      return FALSE;
+    }
+
+  return TRUE;
 }
                                   
 
@@ -1712,15 +1830,17 @@ thunar_renamer_dialog_set_standalone (ThunarRenamerDialog *renamer_dialog,
  * @files             : the list of #ThunarFile<!---->s to rename.
  * @standalone        : whether the dialog should appear like a standalone
  *                      application instead of an integrated renamer dialog.
+ * @startup_id        : startup id to set on the window or %NULL.
  *
  * Convenience function to display a #ThunarRenamerDialog with
  * the given parameters.
  **/
 void
-thunar_show_renamer_dialog (gpointer    parent,
-                            ThunarFile *current_directory,
-                            GList      *files,
-                            gboolean    standalone)
+thunar_show_renamer_dialog (gpointer     parent,
+                            ThunarFile  *current_directory,
+                            GList       *files,
+                            gboolean     standalone,
+                            const gchar *startup_id)
 {
   ThunarApplication *application;
   GdkScreen         *screen;
@@ -1755,6 +1875,10 @@ thunar_show_renamer_dialog (gpointer    parent,
                          "standalone", standalone,
                          NULL);
 
+  /* set the dialogs startup id if available */
+  if (startup_id != NULL && *startup_id != '\0')
+    gtk_window_set_startup_id (GTK_WINDOW (dialog), startup_id);
+
   /* check if we have a toplevel window */
   if (G_LIKELY (window != NULL && GTK_WIDGET_TOPLEVEL (window)))
     {
